<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/

// No direct access
defined('_JEXEC') or die('Restricted access');

$libpath = JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'plugin' . DS;
require_once($libpath . 'CrBcPlugin.php');

abstract class CrBcAdminPlugin extends CrBcPlugin
{
    function __construct(){
        parent::__construct();
        $this->folder = 'admin';
        
        JFactory::getLanguage()->load('breezingcommerce', JPATH_SITE . DS . 'media' . DS . 'breezingcommerce' . DS . 'plugins' . DS . $this->type . DS . $this->name . DS . $this->folder);
    }

    function setDetailsView($toolbarItems = array()){
        $this->setToolbar($toolbarItems);
        parent::setDetailsView($toolbarItems);
    }

    function setListView($toolbarItems = array()){
        $this->setToolbar($toolbarItems);
        parent::setListView($toolbarItems);
    }

    function setToolbar(array $items){
        foreach($items As $item){
            switch($item){
                case 'save':
                JToolBarHelper::save('plugin_save');
                break;
                case 'apply':
                JToolBarHelper::apply('plugin_apply');
                break;
                case 'cancel':
                JToolBarHelper::cancel('plugin_cancel','Close');
                break;
                case 'publish';
                JToolBarHelper::publishList('plugin_publish');
                break;
                case 'unpublish';
                JToolBarHelper::unpublishList('plugin_unpublish');
                break;
                case 'delete';
                JToolBarHelper::deleteList(JText::_('Are you sure you want to delete the selected items'), 'plugin_remove');
                break;
                //case 'edit';
                //JToolBarHelper::editListX('plugin_edit');
                //break;
                case 'add';
                JToolBarHelper::addNew('plugin_add');
                break;
            }
        }
    }

    function delete($ids, $where = '')
    {
        $cids = $ids;
        JArrayHelper::toInteger($cids);
        $row = $this->getTable();

        $this->beforeDeleteRows($cids);
        
        foreach($cids as $cid) {
            
            $this->beforeDeleteRow($cid);
            
            if (!$row->delete( $cid )) {
                //$this->setError( $row->getErrorMsg() );
                return false;
            }
            $this->afterDeleteRow($cid);
        }

        $row->reorder($where);

        $this->afterDeleteRows($ids);
        
        return true;
    }
    
    function setPublished(array $ids, $table = '', $id = 'identity'){
        if($table == ''){
            $table = $this->table;
        }
        JArrayHelper::toInteger($ids);
        JFactory::getDBO()->setQuery( ' Update '.$table.' '.
                '  Set published = 1 Where '.$id.' In ( ' . implode(',',$ids)  . ')');
        JFactory::getDBO()->query();
    }

    function setUnpublished(array $ids, $table = '', $id = 'identity'){
        if($table == ''){
            $table = $this->table;
        }
        JArrayHelper::toInteger($ids);
        JFactory::getDBO()->setQuery( ' Update '.$table.' '.
                '  Set published = 0 Where '.$id.' In ( ' . implode(',',$ids)  . ')');
        JFactory::getDBO()->query();
    }

    function saveOrder($ids, $order, $where = '', $alt_table_classname = '', $alt_table_path = '')
    {
        $items	= $ids;
        JArrayHelper::toInteger($items);

        $total		= count( $items );
        $row		= $this->getTable($alt_table_classname, $alt_table_path);
        $groupings	= array();

        JArrayHelper::toInteger($order);

        // update ordering values
        for( $i=0; $i < $total; $i++ ) {
            $row->load( $items[$i] );
            if ($row->ordering != $order[$i]) {
                $row->ordering = $order[$i];
                if (!$row->store()) {
                    $this->setError($row->getError());
                    return false;
                }
            } // if
        } // for

        $row->reorder($where);
    }

    function move($direction, $id, $where = '', $alt_table_classname = '', $alt_table_path = ''){

      $db = JFactory::getDBO();
      $mainframe = JFactory::getApplication();

      $row = $this->getTable($alt_table_classname, $alt_table_path);

      if (!$row->load($id)) {
         //$this->setError($db->getErrorMsg());
         return false;
      }

      if (!$row->move( $direction, $where )) {
         //$this->setError($db->getErrorMsg());
         return false;
      }

      return true;
   }



    private function buildOrderBy() {
        $mainframe = JFactory::getApplication();
        $option = 'com_breezingcommerce';

        $orderby = '';
        $filter_order     = JFactory::getSession()->get('filter_order', '', $this->session_namespace);
        $filter_order_Dir = JFactory::getSession()->get('filter_order_Dir', '', $this->session_namespace);

        /* Error handling is never a bad thing*/
        if(!empty($filter_order) && !empty($filter_order_Dir) ) {
            $orderby = " ORDER BY `".$filter_order."` ".$filter_order_Dir;
        }

        return $orderby;
    }

    function buildQuery(){
        $where = '';
        
        if($this->table == ''){
            throw new Exception('List table member empty. Please specify a table.');
        }

        $filter_state = '';
        if(JFactory::getSession()->get('filter_state', '', $this->session_namespace) == 'P' || JFactory::getSession()->get('filter_state', '', $this->session_namespace) == 'U')
        {
            $published = 0;
            if(JFactory::getSession()->get('filter_state', '', $this->session_namespace) == 'P')
            {
                $published = 1;
            }

            $filter_state .= ' published = ' . $published;
        }

        if($filter_state != '')
        {
            $where = ' Where ';
        }

        return 'Select SQL_CALC_FOUND_ROWS * From '.$this->table.' ' . $where . ' ' . $filter_state . ' ' . $this->buildOrderBy();
    }

    function getData($limitstart = 0, $limit = 0)
    {
        // Lets load the data if it doesn't already exist
        if (empty( $this->data ))
        {
            $query = $this->buildQuery();
            $this->data = $this->getList( $query, JFactory::getSession()->get('limitstart', '0', $this->session_namespace), JFactory::getSession()->get('limit', '20', $this->session_namespace) );
        }

        return $this->data;
    }

    function getTotal() {
        if (empty($this->total)) {
            $query = $this->buildQuery();
            $this->total = $this->getListCount($query);
        }
        return $this->total;
    }

    function getListCount( $query )
    {
        JFactory::getDBO()->setQuery( $query );
        JFactory::getDBO()->query();

        return JFactory::getDBO()->getNumRows();
    }

    function getList( $query, $limitstart=0, $limit=0 )
    {
        JFactory::getDBO()->setQuery( $query, JFactory::getSession()->get('limitstart', '0', $this->session_namespace), JFactory::getSession()->get('limit', '20', $this->session_namespace) );
	$result = JFactory::getDBO()->loadObjectList();
        return $result;
    }

    function getPagination() {
        // Load the content if it doesn't already exist
        if (empty($this->pagination)) {
            jimport('joomla.html.pagination');
            $this->pagination = new JPagination($this->getTotal(), JFactory::getSession()->get('limitstart', '0', $this->session_namespace), JFactory::getSession()->get('limit', '20', $this->session_namespace));
        }
        return $this->pagination;
    }
}