<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/
defined( '_JEXEC' ) or die( 'Restricted access' );

$libpath = JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'plugin' . DS;
require_once($libpath . 'CrBcAShippingAdminPlugin.php');
require_once($libpath . 'CrBcShippingAdminPlugin.php');

class CrBc_Plugins_Shipping_Flatrate_Admin extends CrBcAShippingAdminPlugin implements CrBcShippingAdminPlugin
{
    private $cart = null;

    public function  __construct() {
        
        require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'classes'.DS.'CrBcCart.php');
        require_once(JPATH_SITE.'/administrator/components/com_breezingcommerce/classes/CrBcPane.php');
        
        // always call the parent constructor and always call it _first_
        parent::__construct();
        // define the default table for built-in list/details view
        $this->table = '#__breezingcommerce_plugin_flatrate';
    }
    
    public function beforeStore($data){
        
        $data['from_total_net'] = doubleval($data['from_total_net']);
        
        if( $data['from_total_net'] < 0 ){
            
            $data['from_total_net'] = 0;
        }
        
        $data['to_total_net'] = doubleval($data['to_total_net']);
        
        if( $data['to_total_net'] < 0 ){
            
            $data['to_total_net'] = 0;
        }
        
        return $data;
    }

    public function afterStore($data){
        if(isset($data['key_translation']) && $data['key_translation'] != ''){
            $fields = array();
            $fields['title'] = trim($data['key_translation']);
            CrBcHelpers::storeTranslation($fields, $data[$this->identity_column], 'plugin_shipping_flatrate');
        }
    }
    
    public function afterDeleteRow($id) {
        $db = JFactory::getDbo();
        $db->setQuery("Delete From #__breezingcommerce_translations Where item_id = ".intval($id)." And type = 'plugin_shipping_flatrate'");
        $db->query();
    }
    
    /**
     * Called on render HTML
     */
    function display(){
        switch(JRequest::getWord('plugin_view','')){
            case '':
            case 'list':
                $this->setListView(array('add', 'delete', 'publish', 'unpublish' ));
                // toolbar items to get back to the main plugins list
                JToolBarHelper::divider();
                JToolBarHelper::cancel('cancel','Close');
            break;
            case 'details':
                // cancel in details view will bring you back to the list view
                $this->setDetailsView(array('apply', 'save', 'cancel'));
            break;
        }
    }

    /**
     * Overriden from CrBcAdminPlugin to display the taxclass id selection
     * @param array $toolbarItems
     */
    function setDetailsView($toolbarItems = array()){

        $this->setToolbar($toolbarItems);
        $this->template = 'details';

        $db = JFactory::getDBO();

        $db->setQuery("Select * From #__breezingcommerce_producttaxclasses Order By `name`");
        $taxclasses = $db->loadObjectList();
        $this->assignRef('taxclasses', $taxclasses);
 
        $db->setQuery("Select * From " . $this->table . " Where `" . $this->identity_column . "` = " . $db->Quote(JRequest::getVar($this->identity_column,'0')));
        $row = $db->loadObject();

        if(!($row instanceof stdClass)){
            $row = new stdClass();
            $row->producttaxclass_id = 0;
            $row->key = '';
            $row->rate = '';
            $row->identity = 0;
            $row->published = false;
            $row->ordering = 0;
        }
        
        
        $id = $this->identity_column;
        $db->setQuery("Select * From #__breezingcommerce_translations Where item_id = ".$row->$id." And type = 'plugin_shipping_flatrate' And language_code = ".$db->quote( JFactory::getSession()->get('com_breezingcommerce.selected_backend_language','') ));
        $info_translation = $db->loadObject();
        $row->key_translation = '';
        if($info_translation !== null){
            $row->key_translation = $info_translation->title;
        }

        $this->assignRef('entity', $row);
    }

    /**
     * Init will be called by the order details model
     * to deal with interface methods.
     *
     * @param CrBcCart $cart
     */
    function init($subject = NULL){
        if( !($subject instanceof CrBcCart) ){ throw new Exception('init subject must be of type CrBcCart'); }
        $this->cart = $subject;
    }

    /**
     * Determines the best available options for this shipping plugin
     *
     * Perfect place to ask shipping services in the background.
     *
     * @param CrBcCart $cart
     */
    function determineBestShippingOptions(){
        // nothing, flatrate doesn't care
    }
    /**
     * Returns a string that renders the available shipping options.
     *
     * The string will be rendered in the Order Details view in the backend.
     *
     * Leave the return value an empty string if no option is available for your shipping plugin.
     *
     * The $selected_key will be stored in the order and returned to this method
     *
     * The selection is mandatory, so it could help when there is a default element with the value "0" (zero)
     * to provide the validator.
     *
     *
     * Only use integers for the option ids!
     *
     * @param mixed $selected_key
     * @return string
     */
    function onDisplayOptions($selected_key){
        
        $db = JFactory::getDBO();
        $db->setQuery("Select * From #__breezingcommerce_plugin_flatrate Where published = 1 Order By `rate`");
        $options = $db->loadAssocList();
        if(count($options)){
            $out = '<select name="'.$this->getOptionElementName().'">'."\n";
            $out .= '<option value="0"> - '.JText::_('COM_BREEZINGCOMMERCE_PLUGIN_FLATRATE_SELECT_ONE').' - </option>';
            foreach($options As $option){
                CrBcCart::$exclude_from_no_tax_list[] = $option['producttaxclass_id'];
                $result = CrBcCart::getPrice($option['rate'], $this->cart->currency_conversion_rate, $option['producttaxclass_id'], 1, $this->cart->cart['customer_id']);
                //$result = CrBcCart::getItemPrice($option['rate'], $option['producttaxclass_id'], array(), array(), 1, $this->cart->cart['customer_id'], false, true);
                $out .= '<option value="'.$option['identity'].'"'.($option['identity'] == $selected_key ? ' selected="selected"' : '').'>'.$option['key'].' ('.$this->cart->formatPrice($result['gross']).')</option>'."\n";
            }

            return $out."\n".'</select>'."\n";
        }

        return '';
    }
    
    function validateSelectedOption($option, $order_id){
        if(intval($option) <= 0){
            JFactory::getApplication()->enqueueMessage(JText::_('COM_BREEZINGCOMMERCE_PLUGIN_FLATRATE_SHIPPING_PLEASE_SELECT_AN_OPTION'), 'error');
            return JText::_('COM_BREEZINGCOMMERCE_PLUGIN_FLATRATE_SHIPPING_PLEASE_SELECT_AN_OPTION');
        }
        return '';
    }
    
    /**
     * Called if this shipping plugin was choosen.
     *
     * @param int $option
     * @param int $order_id
     */
    function onOptionUpdate($option, $order_id){
        // nothing required here
    }

    /**
     * Returns the name of the option selection form element you are using
     * in onDisplayOptions()
     *
     * @return string
     */
    function getOptionElementName(){
        return 'crbc_shipping_flatrate';
    }

    /**
     * Returns the selected product taxclass id for the selected shipping option.
     * The shipping options screen should provde a select list to gather the id.
     *
     * Available product taxclass ids can be retrieved using:
     *
     * $db->setQuery("Select * From #__breezingcommerce_producttaxclasses Order By `name`");
     * $taxclasses = $this->_db->loadObjectList();
     *
     * It is up to the plugin developer to render the taxclass selection!
     *
     * @param int $option
     */
    function getProductTaxclassId($option){
       $db = JFactory::getDBO();
       $db->setQuery("Select producttaxclass_id From #__breezingcommerce_plugin_flatrate Where published = 1 And identity = ".intval($option));
       $result = $db->loadResult();
       if($result){
           return $result;
       }
       return 0;
    }
    /**
     * Returns the price as float/double based on the option given
     *
     * Should NOT return the formatted price as it may be used for further calculations.
     *
     * The price should be returned for the default price.
     * So if you are retrieving the price from a service like canada post, convert it to the shop's default currency!
     *
     * @param int $option
     * @return float
     */
    function getPrice($option){
       $db = JFactory::getDBO();
       $db->setQuery("Select rate From #__breezingcommerce_plugin_flatrate Where published = 1 And identity = ".intval($option));
       $result = $db->loadResult();
       if($result){
           return $result * $this->cart->currency_conversion_rate;
       }
       return 0;
    }

    /**
     * Returns the name of the selected shipping rate
     *
     * @param int $option
     * @return string
     */
    function getName($option){
       $db = JFactory::getDBO();
       $db->setQuery("Select `key` From #__breezingcommerce_plugin_flatrate Where published = 1 And identity = ".intval($option));
       $result = $db->loadResult();
       if($result){
           return $result;
       }
       return JText::_('COM_BREEZINGCOMMERCE_PLUGIN_FLATRATE_UNKNOWN');
    }

    public function getPluginDisplayName(){
        return JText::_('COM_BREEZINGCOMMERCE_PLUGIN_FLATRATE');
    }
}
