<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/

// No direct access

defined( '_JEXEC' ) or die( 'Restricted access' );

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');
jimport( 'joomla.application.component.model' );
require_once(JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce'.DS.'classes'.DS.'CrBcCart.php');

class BreezingcommerceModelOrder extends JModelLegacy
{
    private $_cart = null;
    private $_cart_instance = null;
    private $_cart_items = null;
    private $_plugins = array();
    private $_payment_plugins = array();
    private $_customer = null;

    function  __construct($config)
    {
        parent::__construct();

        $mainframe = JFactory::getApplication();
        $option = 'com_breezingcommerce';

        $array = JRequest::getVar('cid',  0, '', 'array');
        $this->setId((int)$array[0]);
        if(JRequest::getInt('id',0)>0){
            $this->setId(JRequest::getInt('id',0));
        }

        $search = $mainframe->getUserStateFromRequest("$option.order_tabStartOffset".$this->getId(), 'tabStartOffset', '', '0');
        $this->setState('order_tabStartOffset'.$this->getId(), $search);

        
        $cart = new CrBcCart( array() ); // we don't want the cart from the session, so it will be empty
        $this->_cart = $cart->restoreCart($this->_id); // we want to restore the cart from database
        
        $libpath = JPATH_ADMINISTRATOR . DS . 'components' . DS . 'com_breezingcommerce' . DS . 'classes' . DS . 'plugin' . DS;
        require_once($libpath . 'CrBcPlugin.php');

        // SHIPPING PLUGIN INIT - BEGIN
        $this->_db->setQuery("Select * From #__breezingcommerce_plugins Where published = 1 And type = 'shipping' Order By `ordering`");
        $this->_plugins = $this->_db->loadAssocList();

        foreach($this->_plugins As $plugin){
            $plugin_instance = CrBcPlugin::getPluginInstance($plugin['name'], $plugin['type'], 'admin');
            if ($plugin_instance instanceof CrBcShippingAdminPlugin) {
                $plugin_instance->init($cart);
                $plugin_instance->determineBestShippingOptions();
            }
        }
        // SHIPPING PLUGIN INIT - END

        // PAYMENT PLUGIN INIT - BEGIN
        $this->_db->setQuery("Select * From #__breezingcommerce_plugins Where published = 1 And type = 'payment' Order By `ordering`");
        $this->_payment_plugins = $this->_db->loadAssocList();

        foreach($this->_payment_plugins As $plugin){
            $plugin_instance = CrBcPlugin::getPluginInstance($plugin['name'], $plugin['type'], 'admin');
            if ($plugin_instance instanceof CrBcPaymentAdminPlugin) {
                $plugin_instance->init($cart);
            }
        }
        // PAYMENT PLUGIN INIT - END
        
        // CART PLUGINS INIT - BEGIN
        $this->_db->setQuery("Select * From #__breezingcommerce_plugins Where published = 1 And type = 'cart' Order By `ordering`");
        $cart_plugins = $this->_db->loadAssocList();

        $cart_plugin_instances = array();
        foreach( $cart_plugins As $plugin ){
            $plugin_instance = CrBcPlugin::getPluginInstance($plugin['name'], $plugin['type'], 'admin');
            if($plugin_instance instanceof CrBcCartAdminPlugin){
                if($plugin_instance->isSuitable() !== false){
                    $plugin_instance->init($cart);
                }
            }
        }
        // CART PLUGINS INIT - END
        
        $this->_cart_items = $cart->getItems(true);
        $this->_cart_instance = $cart;

        if(isset($this->_cart_instance->cart['customer_id']) && $this->_cart_instance instanceof CrBcCart){
            $this->_db->setQuery("Select * From #__breezingcommerce_customers Where id = " . intval($this->_cart_instance->cart['customer_id']));
            $this->_customer = $this->_db->loadObject();
        }

        $limit = $mainframe->getUserStateFromRequest('global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
        $limitstart = JRequest::getVar('limitstart', 0, '', 'int');

        // In case limit has been changed, adjust it
        $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

        $this->setState('limit', $limit);
        $this->setState('limitstart', $limitstart);

        $filter_order     = $mainframe->getUserStateFromRequest(  $option.'orderitems_filter_order', 'filter_order', 'ordering', 'cmd' );
        $filter_order_Dir = $mainframe->getUserStateFromRequest( $option.'orderitems_filter_order_Dir', 'filter_order_Dir', 'asc', 'word' );

        $this->setState('orderitems_filter_order', $filter_order);
        $this->setState('orderitems_filter_order_Dir', $filter_order_Dir);

        $filter_state = $mainframe->getUserStateFromRequest( $option.'orderitems_filter_state', 'filter_state', '', 'word' );
        $this->setState('orderitems_filter_state', $filter_state);

    }
    
    function getId(){
        return $this->_id;
    }

    function log($type, $message){
        $this->_db->setQuery("Insert Into #__breezingcommerce_order_log (`order_id`,`type`,`message`,`user`,`creation_date`) Values (".intval($this->_id).", ".$this->_db->Quote($type).",".$this->_db->Quote($message).",".$this->_db->Quote(JFactory::getUser()->get('username','unknown') . ' ('.JFactory::getUser()->get('id',0).')').", '".JFactory::getDate()->toSql()."')");
        $this->_db->query();
    }

    function getLog(){
        $this->_db->setQuery("Select * From #__breezingcommerce_order_log Where order_id = " . $this->_id . " Order By `creation_date` Desc");
        return $this->_db->loadObjectList();
    }

    function copy($copied_items = null)
    {
        $insertid = 0;
        $insertIds = array();
        $items = $this->getData();

        foreach($items As $item){
            if($copied_items === null || (is_array($copied_items) && in_array($item->id, $copied_items))){
                $irow = $this->getTable('orderitems');
                $this->_db->setQuery("Select id From #__breezingcommerce_products Where id = " . $item->product_id);
                if($this->_db->loadResult() && $irow->load($item->id)){
                    unset($irow->id);
                    $irow->history = 0;
                    $irow->creation_date = JFactory::getDate()->toSql();
                    $irow->verified = 0;
                    $this->_db->insertObject('#__breezingcommerce_order_items', $irow);
                    $_insertid = $this->_db->insertid();
                    $insertIds[] = $_insertid;
                    if(is_array($copied_items) && in_array($item->id, $copied_items)){
                        $irow->delete($item->id);
                        $this->log('general', 'Moved splitted item ' . $item->id . ' into new order. New item id: ' . $_insertid);
                    }
                }
            }
        }

        if(count($insertIds)){
            
            $order = CrBcCart::getOrder(
                                    $this->_id, 
                                    $this->_cart_instance, 
                                    $this->_cart, 
                                    $this->_cart_items, 
                                    $this->_customer,
                                    $this->getData(),
                                    $this->_plugins,
                                    $this->_payment_plugins
                            );

            $this->_db->setQuery("Select id From #__breezingcommerce_customers Where id = " . $order->customer_id);
            if(!$this->_db->loadResult()){
                $this->_db->setQuery("Delete From #__breezingcommerce_order_items Where id In (".implode(',',$insertIds).")");
                $this->_db->query();
                return 0;
            }

            $orow = $this->getTable('order');
            if($orow->load($this->_id)){
                $old_id = $orow->id;
                unset($orow->id);
                unset($orow->order_number);
                $orow->history = 0;
                $orow->checked_out = 0;
                $orow->invoice_created = 0;
                $orow->cancelled = 0;
                $orow->order_status_id = 0;
                $orow->send_notification = 1;
                $orow->plugin_id = 0;
                $orow->selected_shipping_id = 0;
                $orow->creation_date = JFactory::getDate()->toSql();
                $orow->paid = 0;
                $orow->payment_date = '0000-00-00 00:00:00';
                $this->_db->insertObject('#__breezingcommerce_orders', $orow);
                $insertid = $this->_db->insertid();
                if($insertid){
                    $this->_db->setQuery("Update #__breezingcommerce_order_items Set order_id = " . $insertid . " Where id In (".implode(',',$insertIds).")");
                    $this->_db->query();
                    $this->log('general', 'Copied from orderid ' . $old_id, $insertid);
                    $this->log('general', 'Copy created, new orderid ' . $insertid);
                }
            }
        }

        return $insertid;
    }

    function setPublished()
    {
        if (empty( $this->_data )) {
            $this->_db->setQuery( ' Update #__breezingcommerce_orders '.
                        '  Set published = 1 Where id = '.$this->_id );
            $this->_db->query();
        }
    }

    function setUnpublished()
    {
       if (empty( $this->_data )) {
           $this->_db->setQuery( ' Update #__breezingcommerce_orders '.
                        '  Set published = 0 Where id = '.$this->_id );
           $this->_db->query();
       }
    }

    function setListVerified()
    {
        $items	= JRequest::getVar( 'cid', array(), 'post', 'array' );
        JArrayHelper::toInteger($items);
        if (count($items)) {
            //$this->_db->setQuery( ' Update #__breezingcommerce_order_items '.
            //            '  Set verified = 1 Where id In ( '.implode(',', $items) . ')' );
            //$this->_db->query();
            
            foreach($items As $item){
                CrBcCart::verifyItem($item, true, JFactory::getDate('now', 'UTC')->toSql());
                CrBcCart::verifyFilesByOrderItemId($item);
            }
            
            $this->log('general', 'Verified item(s) ' . implode(',', $items));
            
        }
    }

    function setListUnverified()
    {
       $items	= JRequest::getVar( 'cid', array(), 'post', 'array' );
        JArrayHelper::toInteger($items);
        if (count($items)) {
            // because there are date in verified items in play, we won't touch anything else than the verified flag
            $this->_db->setQuery( ' Update #__breezingcommerce_order_items '.
                        '  Set verification_expiration_date = "0000-00-00 00:00:00", verified = 0 Where id In ( '.implode(',', $items) . ')' );
            $this->_db->query();
            $this->log('general', 'Unverified item(s) ' . implode(',', $items));
            foreach($items As $item){
                
                CrBcCart::unverifyFilesByOrderItemId($item);
            }
        }
    }

    function setListPublished()
    {
        $items	= JRequest::getVar( 'cid', array(), 'post', 'array' );
        JArrayHelper::toInteger($items);
        if (count($items)) {
            $this->_db->setQuery( ' Update #__breezingcommerce_order_items '.
                        '  Set published = 1 Where id In ( '.implode(',', $items) . ')' );
            $this->_db->query();
            $this->log('general', 'Unset backorder status for item(s) ' . implode(',', $items));
        }
    }

    function setListUnpublished()
    {
       $items	= JRequest::getVar( 'cid', array(), 'post', 'array' );
        JArrayHelper::toInteger($items);
        if (count($items)) {
            $this->_db->setQuery( ' Update #__breezingcommerce_order_items '.
                        '  Set published = 0 Where id In ( '.implode(',', $items) . ')' );
            $this->_db->query();
            $this->log('general', 'Set backorder status for item(s) ' . implode(',', $items));
        }
    }

    /*
     * MAIN DETAILS AREA
     */

    /**
     *
     * @param int $id 
     */
    function setId($id) {
        // Set id and wipe data
        $this->_id      = $id;
        $this->_data    = null;
    }

    function isVirtualOrder(){
        return CrBcCart::isVirtualOrder($this->_cart_items);
    }
    
    function getCartInstance(){
        return $this->_cart_instance;
    }

    function getOrder()
    {
        return CrBcCart::getOrder(
                    $this->_id, 
                    $this->_cart_instance, 
                    $this->_cart, 
                    $this->_cart_items, 
                    $this->_customer,
                    $this->getData(),
                    $this->_plugins,
                    $this->_payment_plugins
            );
    }
    
    function getData()
    {
        return CrBcCart::getData($this->_id, $this->_cart_items, -1, -1, JFactory::getApplication()->isAdmin());
        //return CrBcCart::getData($this->_id, $this->_cart_items, $this->getState('limitstart'), $this->getState('limit'), JFactory::getApplication()->isAdmin());
    }

    function store()
    {
        $row = $this->getTable();
        
        $data = JRequest::get( 'post' );

        //if( isset($data['id']) && intval($data['id']) > 0 ){
        //
        //    $row->load($data['id']);
        //}
        
        /////
        // NOTIFY SHIPPING PLUGINS ABOUT CHANGES

        $shipping_costs = 0;
        $shipping_plugin = '';
        $shipping_name = '';
        $payment_plugin = '';
        $payment_plugin_info = '';
        
        $orow = $this->getTable('order');
        $orow->load($this->_id);

        if(!isset($data['paid'])){
            $data['paid'] = $orow->paid;
        }else{
            $data['payment_date'] = JFactory::getDate()->toSql();
        }
        
        if(!$orow->invoice_created){

            $is_virtual_order = $this->isVirtualOrder();
            $found = false;
            if(!$is_virtual_order){
                foreach ($this->_plugins As $plugin) {
                    if ($plugin['id'] == $data['plugin_id']) {
                        $plugin_instance = CrBcPlugin::getPluginInstance($plugin['name'], $plugin['type'], 'admin');
                        $class = CrBcPlugin::getPluginClass($plugin['name'], $plugin['type'], 'admin');
                        if ($plugin_instance instanceof CrBcShippingAdminPlugin && !$plugin_instance->validateSelectedOption(JRequest::getInt($plugin_instance->getOptionElementName(), 0), $this->_id)) {
                            $plugin_instance->init($this->_cart_instance);
                            $plugin_instance->onOptionUpdate(
                                    JRequest::getInt($plugin_instance->getOptionElementName(), 0),
                                    $this->_id
                            );
                            $data['selected_shipping_id'] = JRequest::getInt($plugin_instance->getOptionElementName(), 0);
                            $shipping_product_tax_class_id = $plugin_instance->getProductTaxclassId($data['selected_shipping_id']);
                            CrBcCart::$exclude_from_no_tax_list[] = $shipping_product_tax_class_id;
                            $shipping_costs = $plugin_instance->getPrice($data['selected_shipping_id']);
                            $price_calc = CrBcCart::getPrice($shipping_costs, 1, $shipping_product_tax_class_id, 1, $this->_cart_instance->cart['customer_id']);
                            $shipping_costs = $price_calc['gross'];
                            $shipping_name = $plugin_instance->getName($data['selected_shipping_id']);
                            $shipping_plugin = $plugin_instance->getPluginDisplayName();
                            $found = true;
                        }
                        break;
                    }
                }
            }else{
                $shipping_costs = 0;
                $shipping_name = JText::_('COM_BREEZINGCOMMERCE_ELECTRONIC_DELIVERY');
                $shipping_plugin = JText::_('COM_BREEZINGCOMMERCE_NO_SHIPPING');
            }

            if (!$found && !$is_virtual_order) {
                return 1; // signals controller to return error message (no shipping method selected)
            }

            $found = false;
            foreach ($this->_payment_plugins As $payment_plugin) {
                if ($payment_plugin['id'] == $data['payment_plugin_id']) {
                    $plugin_instance = CrBcPlugin::getPluginInstance($payment_plugin['name'], $payment_plugin['type'], 'admin');
                    $class = CrBcPlugin::getPluginClass($payment_plugin['name'], $payment_plugin['type'], 'admin');
                    $payment_plugin = $plugin_instance->getPluginDisplayName();
                    if(!$orow->paid){
                        $payment_plugin_info = $plugin_instance->getPaymentInfo();
                    }else{
                        $payment_plugin_info = $plugin_instance->getAfterPaymentInfo();
                    }
                    $found = true;
                    break;
                }
            }

            if (!$found) {
                $order_info = CrBcCart::getOrder(
                                    $this->_id, 
                                    $this->_cart_instance, 
                                    $this->_cart, 
                                    $this->_cart_items, 
                                    $this->_customer,
                                    $this->getData(),
                                    $this->_plugins,
                                    $this->_payment_plugins
                            );
                
                if($order_info->price_net + $order_info->taxes + $order_info->shipping_costs > 0){
                    return 3; // signals controller to return error message (no payment method selected)
                }else{
                    $payment_plugin = JText::_('COM_BREEZINGCOMMERCE_NO_PAYMENT');
                    $payment_plugin_info = JText::_('COM_BREEZINGCOMMERCE_NOT_AVAILABLE');
                }
                
            }
            
            $orow->history_shipping_plugin = $shipping_plugin;
            $orow->history_shipping_name = $shipping_name;
            $orow->history_shipping_costs = $shipping_costs;
            $orow->history_payment_plugin = $payment_plugin;
            $orow->history_payment_plugin_info = $payment_plugin_info;
            $orow->store();
        }
        
        if( $data['paid'] == 1 ){
            CrBcCart::verifyItems($this->_id, false);
        }

        //////
        //
        // ON NOTIFICATION MESSAGE
        //
        
        if($data['notification_message'] && $data['notification_message_subject']){

            $email = '';

            $this->_db->setQuery("Select c.* From #__breezingcommerce_customers As c, #__users As u Where (u.id = c.userid Or c.userid = 0) And c.id = " . intval($orow->customer_id));
            $customer_result = $this->_db->loadAssoc();

            if (is_array($customer_result)) {
                if ($customer_result['userid'] == 0) {
                    $email = $customer_result['email'];
                } else {
                    $this->_db->setQuery("Select email From #__users Where id = " . intval($customer_result['userid']));
                    $email = $this->_db->loadResult();
                }
            } else {
                if($orow->history_customer){
                    $cust = @unserialize($orow->history_customer);
                    $email = $cust->email;
                }
            }

            $this->log('general', 'Notification saved:'."\n".$data['notification_message']);

            if($email && isset($data['notification_message_customer']) && $data['notification_message_customer'] == 'true'){
                
                $subject = $data['notification_message_subject'];
                $body = $data['notification_message'];
                
                JFactory::getMailer()->sendMail(
                    JFactory::getApplication()->getCfg('mailfrom'), 
                    JFactory::getApplication()->getCfg('fromname'), 
                    $email, 
                    $subject, 
                    $body
            );
                
                $this->log('general', 'Notification mail sent');
            }
        }

        //////
        // SCAN FOR CUSTOM PRICES
        // IT WILL OVERRIDE ANY PRICE FOR THE SELECTED ITEM
        //
        
        if(isset($data['invoice_created']) && $data['invoice_created'] == 0){
            $prices = JRequest::getVar('prices', array(), 'post', 'array');
            $price_overrides = JRequest::getVar('price_overrides', array(), 'post', 'array');
            $irow = $this->getTable('orderitems');
            $items = $this->getData();
            foreach ($items As $item) {
                if(isset($price_overrides[$item->id]) && isset($prices[$item->id]) && is_numeric($prices[$item->id])){
                    if($item->history == 1){
                        if ($irow->load($item->id)) {
                            $price = CrBcCart::getItemPrice( $prices[$item->id], $this->_cart_instance->currency_conversion_rate, $irow->producttaxclass_id, array(), array(), 1, $this->_cart_instance->cart['customer_id'], $irow->is_combination == 1 ? true :  false);
                            $price_single = CrBcCart::getItemPrice($prices[$item->id], $this->_cart_instance->currency_conversion_rate, $irow->producttaxclass_id, array(), array(), 1, $this->_cart_instance->cart['customer_id'], $irow->is_combination == 1 ? true : false);
                            $old_price = $irow->history_net;
                            $irow->history_net = $price['net'];
                            $irow->history_gross = $price['gross'];
                            $taxes = 0;
                            $applied_taxes = '';
                            foreach ($price['tax_list'] As $id => $info) {
                                $applied_taxes .= ' - ' . $info['name'] . ' ('.CrBcCart::formatNumber($info['rate']).' %)';
                                $taxes += $info['tax'] <= 0 ? 0 : CrBcCart::round($info['tax']);
                            }
                            $irow->history_taxes = $taxes;
                            $irow->history_applied_taxes = $applied_taxes . ' - ';
                            $irow->history_single_net = $price_single['net'];
                            $irow->history_single_gross = $price_single['gross'];
                            $taxes = 0;
                            $applied_taxes = '';
                            foreach ($price_single['tax_list'] As $id => $info) {
                                $applied_taxes .= ' - ' . $info['name'] . ' ('.CrBcCart::formatNumber($info['rate']).' %)';
                                $taxes += $info['tax'] <= 0 ? 0 : $info['tax'];
                            }
                            $irow->history_single_taxes = $taxes;
                            $irow->history_single_applied_taxes = $applied_taxes . ' - ';
                            $irow->history_discount = $price['off'];
                            $irow->history_single_discount = $price_single['off'];
                            $irow->history_price_tax_list = @serialize($price['tax_list']);
                            $irow->history_price_single_tax_list = @serialize($price_single['tax_list']);

                            $irow->store();

                            $this->log('general', 'Added custom price to order item id ' . $item->id . '. Old price: ' . $old_price . '. New price: ' . $price['net']);
                        }
                    }
                }
            }
        }

        //////
        //  ON CHECKOUT, SPLIT BACKORDERED ITEMS (IF SET TO DO SO)
        //  ALSO FREEZE (PERSIST) THE ITEM DATA

        if(isset($data['checked_out']) && intval($data['checked_out']) == 1){

            $shipping_info = array();
            $shipping_info['shipping_plugin'] = $shipping_plugin;
            $shipping_info['shipping_name'] = $shipping_name;
            $shipping_info['shipping_costs'] = $shipping_costs;
            
            $payment_info = array();
            $payment_info['payment_plugin'] = $payment_plugin;
            $payment_info['payment_plugin_info'] = $payment_plugin_info;
            
            CrBcCart::checkout(
                    $this->_id, 
                    $this->_cart_instance, 
                    $this->_cart_items, 
                    $this->_cart, 
                    $shipping_info, 
                    $payment_info
            );
        }

        /////
        
        //////
        // ON INVOICE CREATION

        if( (isset($data['invoice_created']) && intval($data['invoice_created']) == 1) || (isset($data['cancelled']) && intval($data['cancelled']) == 1)) {
            
            $config = CrBcHelpers::getBcConfig();
        
            // make sure to only load the default backend language
            $lang_tmp = JFactory::getLanguage()->getTag();
            $lang_selected = $lang_tmp;

            switch($config->get('invoice_language','backend')){
                case 'backend':
                    $lang_selected = JComponentHelper::getParams('com_languages')->get('administrator');
                    break;
                case 'frontend':
                    $lang_selected = JComponentHelper::getParams('com_languages')->get('site');
                    break;
                case 'frontend_user':
                    $lang_selected = JFactory::getLanguage()->getTag();
                    break;
            }

            JFactory::getLanguage()->load('com_breezingcommerce', JPATH_BASE, $lang_selected, true);

            
            $customer = $this->_cart_instance->getCustomer();
            
            if( !$customer->address_complete ){
                return 5;
            }
            
            // removing products from stock
            if(isset($data['invoice_created']) && intval($data['invoice_created']) == 1){
                CrBcCart::stockUpdate($this->_id, $this->_cart_items, false,'-');
            }else{
                if($data['cancel_stockback'] == 1){
                    CrBcCart::stockUpdate($this->_id, $this->_cart_items, false,'+');
                }
            }

            $customer_id = 0;
            $order = $this->getTable('order');
            $order_number = '';
            $order_info = CrBcCart::getOrder(
                                    $this->_id, 
                                    $this->_cart_instance, 
                                    $this->_cart, 
                                    $this->_cart_items, 
                                    $this->_customer,
                                    $this->getData(),
                                    $this->_plugins,
                                    $this->_payment_plugins
                            );
            if($order_info->grand_total <= 0){
                return 4;
            }
            
            $date = JFactory::getDate()->toSql();
            
            if ($order->load($this->_id)) {
                $order_number = $order->order_number;
                $customer_id = $order->history_customer_id;
                $shipping_name = $order->history_shipping_name;
                $payment_plugin = $order->history_payment_plugin;
                $payment_plugin_info = $order->history_payment_plugin_info;
            }

            $customer = $this->_customer;
            if(!($customer instanceof stdClass)){
                // retrieve from history
                $customer = @unserialize($order->history_customer);
            }else{
                $this->_db->setQuery("Select `name` From #__breezingcommerce_countries Where id = " . intval($customer->country_id));
                $res = $this->_db->loadObject();
                $customer->country = isset($res->name) ? $res->name : '';

                $this->_db->setQuery("Select `name` From #__breezingcommerce_countries Where id = " . intval($customer->shipping_country_id));
                $res = $this->_db->loadObject();
                $customer->shipping_country = isset($res->name) ? $res->name : '';

                $this->_db->setQuery("Select `name` From #__breezingcommerce_country_regions Where id = " . intval($customer->region_id));
                $res = $this->_db->loadObject();
                $customer->region = isset($res->name) ? $res->name : '';

                $this->_db->setQuery("Select `name` From #__breezingcommerce_country_regions Where id = " . intval($customer->shipping_region_id));
                $res = $this->_db->loadObject();
                $customer->shipping_region = isset($res->name) ? $res->name : '';
            }
            if($customer instanceof stdClass){
                
                if(isset($data['invoice_created']) && intval($data['invoice_created']) == 1){
                    $order->invoice_created = 1;
                    $order_info->invoice_date = $date;
                }else{
                    $order->cancelled = 1;
                    $order_info->cancelled_date = $date;
                }

                CrBcCart::createAndSendInvoice(
                        $customer, 
                        $order, 
                        $order_info, 
                        $this->_cart_instance, 
                        $this->_cart_items, 
                        isset($data['cancelled']) && intval($data['cancelled']) == 1
                );
                
                // revert to original language after all
                JFactory::getLanguage()->load('com_breezingcommerce', JPATH_BASE, $lang_tmp, true);

                $order->store();
                
            }else{
                return 2;
            }
        }

        // ON ORDER STATUS CHANGE, TRIGGER EMAILS IF NECESSARY

        if(!isset($data['send_notification'])){
            $data['send_notification'] = 0;
        }

        $email = '';
        $customer_result = null;
        $result = null;
        $result2 = null;
        $order = CrBcCart::getOrder(
                        $this->_id, 
                        $this->_cart_instance, 
                        $this->_cart, 
                        $this->_cart_items, 
                        $this->_customer,
                        $this->getData(),
                        $this->_plugins,
                        $this->_payment_plugins
                );

        if($order->checked_out){

            if($order->order_status_id != $data['order_status_id'] && intval($data['send_notification']) == 1){
                $results = CrBcCart::getCustomerResults($order, $data['order_status_id'], $customer);
                $email = $results['email'];
                $result = $results['result'];
                $result2 = $results['result2'];
                $customer_result = $results['customer_result'];
            }
        }
        else
        {
            
            // CART PLUGINS INIT - BEGIN
            $this->_db->setQuery("Select * From #__breezingcommerce_plugins Where published = 1 And type = 'cart' Order By `ordering`");
            $cart_plugins = $this->_db->loadAssocList();

            $cart_plugin_instances = array();
            foreach( $cart_plugins As $plugin ){
                $plugin_instance = CrBcPlugin::getPluginInstance($plugin['name'], $plugin['type'], 'admin');
                if($plugin_instance instanceof CrBcCartAdminPlugin){
                    if($plugin_instance->isSuitable() !== false){
                        $plugin_instance->init($this->_cart_instance);
                        $plugin_instance->viewportReceive();
                        $data['tax_disabled'] = $plugin_instance->isTaxDisabled() ? 1 : 0;
                    }
                }
            }
            // CART PLUGINS INIT - END
        }

        //////
        // FINAL STORE AND EMAIL NOTIFICATIONS

        // if checked_out is not set, then it is keeps on being checked_out
        if(!isset($data['checked_out'])){
            $data['checked_out'] = 1;
        }

        if(!isset($data['cancelled'])){
            $data['cancelled'] = 1;
        }

        if(!isset($data['invoice_created'])){
            $data['invoice_created'] = 1;
        }

        if (!$row->bind($data)) {
            $this->setError($this->_db->getErrorMsg());
            return false;
        }

        if (!$row->check()) {
            $this->setError($this->_db->getErrorMsg());
            return false;
        }

        $storeRes = $row->store();

        if (!$storeRes) {
            $this->setError($this->_db->getErrorMsg());
            return false;
        }
        else
        {
            if ($order->checked_out) {
                
                CrBcCart::sendStatusMail($order, $email, $result, $result2, $customer_result, $this->_id);

                if($result2['name']){
                    $this->log('general', 'Status changed to ' . $result2['name']);
                }
            }
        }
        
        //$row->reorder();

        return true;
    }

    function delete()
    {
        $cids = JRequest::getVar( 'cid', array(0), 'post', 'array' );
        JArrayHelper::toInteger($cids);
        $row = $this->getTable();
        foreach($cids as $cid) {
            $row->load($cid);
            //if(!$row->invoice_created){
            if($row->history == 0 && $row->checked_out == 0){
                /*
                $this->_db->setQuery("Select id From #__breezingcommerce_order_items Where order_id = " . $cid);
                $order_items = $this->_db->loadAssocList();
                foreach($order_items As $order_item){
                    if($row->invoice_created){
                        $this->stockUpdate($order_item['id'],'+');
                    }
                }
                 */
                $this->_db->setQuery("Delete From #__breezingcommerce_order_items Where order_id = " . $cid);
                $this->_db->query();
                $this->_db->setQuery("Delete From #__breezingcommerce_order_log Where order_id = " . $cid);
                $this->_db->query();
                if (!$row->delete( $cid )) {
                    $this->setError( $row->getErrorMsg() );
                    return false;
                }
            }else{
                $this->_db->setQuery("Update #__breezingcommerce_orders Set published = 0 Where id = " . $cid);
                $this->_db->query();
            }
        }

        //$row->reorder();

        return true;
    }

    function listDelete()
    {
        $combinations_to_delete = array();
        $cids = JRequest::getVar( 'cid', array(0), 'post', 'array' );
        JArrayHelper::toInteger($cids);
        $orow = $this->getTable('order');
        $orow->load($this->_id);
        foreach ($cids as $cid) {
            if($orow->invoice_created){
                CrBcCart::stockUpdate($this->_id, $this->_cart_items, $cid,'+');
            }
            $this->_db->setQuery("Delete From #__breezingcommerce_order_items Where id = " . $cid);
            $this->_db->query();
            $this->log('general', 'Item ' . $cid . ' deleted');
        }
    }

    function move($direction) {

      $db = JFactory::getDBO();
      $mainframe = JFactory::getApplication();

      $row = $this->getTable('order');

      if (!$row->load($this->_id)) {
         $this->setError($db->getErrorMsg());
         return false;
      }

      if (!$row->move( $direction )) {
         $this->setError($db->getErrorMsg());
         return false;
      }

      return true;
   }

   function listMove($direction) {
      $mainframe = JFactory::getApplication();
      $items = JRequest::getVar( 'cid', array(), 'post', 'array' );
      JArrayHelper::toInteger($items);

      if(count($items)){
          $db = JFactory::getDBO();
          $row = $this->getTable('orderitems');

          if (!$row->load($items[0])) {
             $this->setError($db->getErrorMsg());
             return false;
          }

          if (!$row->move( $direction, 'order_id='.$this->_id )) {
             $this->setError($db->getErrorMsg());
             return false;
          }
      }

      return true;
   }
   
   function _buildQuery()
    {
        return "Select * From #__breezingcommerce_order_items Where order_id = " . $this->_id . "  Order By `creation_date` Desc  ";
    }

   function getTotal() {
        // Load the content if it doesn't already exist
        if (empty($this->_total)) {
            $query = $this->_buildQuery();
            $this->_total = $this->_getListCount($query);
        }
        return $this->_total;
    }

    function getPagination() {
        // Load the content if it doesn't already exist
        if (empty($this->_pagination)) {
            jimport('joomla.html.pagination');
            $this->_pagination = new JPagination($this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
        }
        return $this->_pagination;
    }

    function listSaveOrder()
    {
        $items	= JRequest::getVar( 'cid', array(), 'post', 'array' );
        JArrayHelper::toInteger($items);

        $total		= count( $items );
        $row		= $this->getTable('orderitems');
        $groupings	= array();

        $order		= JRequest::getVar( 'order', array(), 'post', 'array' );
        JArrayHelper::toInteger($order);

        // update ordering values
        for( $i=0; $i < $total; $i++ ) {
            $row->load( $items[$i] );
            if ($row->ordering != $order[$i]) {
                $row->ordering = $order[$i];
                if (!$row->store()) {
                    $this->setError($row->getError());
                    return false;
                }
            } // if
        } // for


        $row->reorder("order_id = " . $this->_id);
    }
}
