<?php
/**
 * @package     BreezingCommerce
 * @author      Markus Bopp
 * @link        http://www.crosstec.de
 * @license     GNU/GPL
*/

// No direct access

defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.application.component.model' );

class BreezingcommerceModelOrders extends JModelLegacy
{
    /**
     * Items total
     * @var integer
     */
    private $_total = null;

    /**
     * Pagination object
     * @var object
     */
    private $_pagination = null;

    function  __construct($config) {
        parent::__construct($config);

        $mainframe = JFactory::getApplication();
        $option = 'com_breezingcommerce';
        
        $search = $mainframe->getUserStateFromRequest("$option.order_search", 'order_search', '', 'string');
        $this->setState('order_search', $search);

        $search = $mainframe->getUserStateFromRequest("$option.order_search_by", 'order_search_by', '', 'string');
        $this->setState('order_search_by', $search);

        // Get pagination request variables
        $limit = $mainframe->getUserStateFromRequest('global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
        $limitstart = JRequest::getVar('limitstart', 0, '', 'int');

        // In case limit has been changed, adjust it
        $limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

        $this->setState('limit', $limit);
        $this->setState('limitstart', $limitstart);

        $filter_order     = $mainframe->getUserStateFromRequest(  $option.'order_filter_order', 'filter_order', 'o.id', 'cmd' );
        $filter_order_Dir = $mainframe->getUserStateFromRequest( $option.'order_filter_order_Dir', 'filter_order_Dir', 'desc', 'word' );

        $this->setState('order_filter_order', $filter_order);
        $this->setState('order_filter_order_Dir', $filter_order_Dir);

        $filter_state = $mainframe->getUserStateFromRequest( $option.'order_filter_state', 'filter_state', '', 'word' );
        $this->setState('order_filter_state', $filter_state);

        $status_state = $mainframe->getUserStateFromRequest( $option.'orders_status_state', 'status_state', '', 'cmd' );
        $this->setState('orders_status_state', $status_state);
    }

    function getStatus(){
        $this->_db->setQuery("Select Distinct `id`,`name` From #__breezingcommerce_order_status Where published = 1 Order By ordering");
        return $this->_db->loadObjectList();
    }

    function setPublished()
    {
        $cids = JRequest::getVar('cid', array(), '', 'array');
        JArrayHelper::toInteger($cids);
        $this->_db->setQuery( ' Update #__breezingcommerce_orders '.
                '  Set published = 1 Where id In ( ' . implode(',',$cids)  . ')');
        $this->_db->query();

    }

    function setUnpublished()
    {
        $cids = JRequest::getVar('cid', array(), '', 'array');
        JArrayHelper::toInteger($cids);
        $this->_db->setQuery( ' Update #__breezingcommerce_orders '.
                '  Set published = 0 Where id In ( ' . implode(',',$cids)  . ')');
        $this->_db->query();
    }

    /*
     *
     * MAIN LIST AREA
     * 
     */

    private function buildOrderBy() {
        $mainframe = JFactory::getApplication();
        $option = 'com_breezingcommerce';

        $orderby = '';
        $filter_order     = $this->getState('order_filter_order');
        $filter_order_Dir = $this->getState('order_filter_order_Dir');

        /* Error handling is never a bad thing*/
        if(!empty($filter_order) && !empty($filter_order_Dir) ) {
            $orderby = ' ORDER BY '.$filter_order.' '.$filter_order_Dir;
        }

        return $orderby;
    }


    /**
     * @return string The query
     */
    private function _buildQuery(){

        $where = '';
        $match = '';
        
        // FILTER ACTIVATED?
        if($this->getState('order_search_by') == 'order_number' && $this->getState('order_search') != '')
        {
            $match = " And o.order_number = " . $this->_db->Quote( $this->getState('order_search' ) ) . " ";
        }
        else if($this->getState('order_search_by') == 'customer_id' && $this->getState('order_search') != '')
        {
            $match = " And ( o.history_customer_id = " . $this->_db->Quote( $this->getState('order_search' ) ) . " Or o.customer_id = " . $this->_db->Quote( $this->getState('order_search' ) ) . " ) ";
        }
        else if($this->getState('order_search_by') == 'customer_name' && $this->getState('order_search') != '')
        {
            $match = " And o.history_customer_name Like " . $this->_db->Quote( '%'.$this->getState('order_search' ) . '%' );
        }
        else if($this->getState('order_search_by') == 'invoice_number' && $this->getState('order_search') != '')
        {
            $match = " And o.invoice_number = " . $this->_db->Quote( $this->getState('order_search' ) );
        }
        else if($this->getState('order_search_by') == 'cancelled_number' && $this->getState('order_search') != '')
        {
            $match = " And o.cancelled_number = " . $this->_db->Quote( $this->getState('order_search' ) );
        }
        else if($this->getState('order_search_by') == 'creation_date' && $this->getState('order_search') != '')
        {
            $match = " And (o.creation_date between " . $this->_db->Quote( $this->getState('order_search' ) . ' 00:00:00' ) . " And " . $this->_db->Quote( $this->getState('order_search' ) . ' 23:59:59' ) . " )";
        }
        else if($this->getState('order_search_by') == 'creation_year' && $this->getState('order_search') != '')
        {
            $match = " And (o.creation_date between " . $this->_db->Quote( $this->getState('order_search' ) . '-01-01 00:00:00' ) . " And " . $this->_db->Quote( $this->getState('order_search' ) . '-12-31 23:59:59' ) . " )";
        }
        else if($this->getState('order_search_by') == 'creation_year_month' && $this->getState('order_search') != '')
        {
            $match = " And (o.creation_date between " . $this->_db->Quote( $this->getState('order_search' ) . '-01 00:00:00' ) . " And " . $this->_db->Quote( $this->getState('order_search' ) . '-31 23:59:59' ) . " )";
        }

        $state = '';
        if($this->getState('orders_status_state') != ''){
            $state .= ' And o.`order_status_id` = ' . $this->_db->Quote($this->getState('orders_status_state')) . ' ';
        }

        $where = " Where o.history_customer_name <> '' And o.published = 1 " . $match . $state;

        return "
            Select SQL_CALC_FOUND_ROWS Distinct o.*, s.name As order_status_name, Sum(i.published) As backorder, Sum(i.history_gross) + o.history_shipping_costs As order_total
                From
                    #__breezingcommerce_orders As o
                Left Join
                    ( 
                      #__breezingcommerce_order_status As s,
                      #__breezingcommerce_order_items As i
                    )
                On
                    (
                        s.id = o.order_status_id
                      And
                        i.order_id = o.id
                    )
            " . $where . $match . ' Group By o.id ' . $this->buildOrderBy();
    }

    /**
    * Gets the currencies
    * @return array List of products
    */
    function getData()
    {
        // Lets load the data if it doesn't already exist
        if (empty( $this->_data ))
        {
            $query = $this->_buildQuery();
            $this->_data = $this->_getList( $query, $this->getState('limitstart'), $this->getState('limit') );
            $this->_db->setQuery("SELECT FOUND_ROWS()");
            $this->_total = $this->_db->loadResult();
            echo $this->_db->getErrorMsg();
        }

        return $this->_data;
    }

    function getTotal() {
        // Load the content if it doesn't already exist
        if (empty($this->_total)) {
            $query = $this->_buildQuery();
            $this->_total = $this->_getListCount($query);
        }
        return $this->_total;
    }

    function getPagination() {
        // Load the content if it doesn't already exist
        if (empty($this->_pagination)) {
            jimport('joomla.html.pagination');
            //$this->_pagination = new JPagination($this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
            $this->_pagination = new JPagination($this->_total, $this->getState('limitstart'), $this->getState('limit') );
        }
        return $this->_pagination;
    }

}
